/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "rtd.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define RTD_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void rtd_cfg_setup ( rtd_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->drdy = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

RTD_RETVAL rtd_init ( rtd_t *ctx, rtd_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return RTD_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, RTD_DUMMY );

    // Input pins

    digital_in_init( &ctx->drdy, cfg->drdy );
    
    return RTD_OK;
}

void rtd_generic_transfer ( rtd_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

void rtd_write_register ( rtd_t *ctx, uint8_t reg_address, uint8_t write_data )
{
    uint8_t tmp[ 2 ];

    reg_address |= 0x80;
    
    tmp[ 0 ] = reg_address;
    tmp[ 1 ] = write_data;

    spi_master_start( &ctx->spi );

    spi_master_write( &ctx->spi, tmp[ 0 ] );
    spi_master_write( &ctx->spi, tmp[ 1 ] );

    spi_master_stop( &ctx->spi );   
}

uint8_t rtd_read_register ( rtd_t *ctx, uint8_t reg_address )
{
    spi_master_transfer_data_t block;
    uint8_t tmp;
    uint8_t rx_data[ 2 ];

    block.tx_buffer = &reg_address;
    block.rx_buffer = rx_data;
    block.tx_length = 1;
    block.rx_length = 2;

    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, &block );
    spi_master_stop( &ctx->spi );

    tmp = rx_data[ 1 ];

    return tmp;
}

uint16_t rtd_read_temperature ( rtd_t *ctx )
{
    uint16_t return_value;

    return_value = rtd_read_register( ctx, 0x01 );
    return_value <<= 8;
    return_value |= rtd_read_register( ctx, 0x02 );

    return return_value;
}

float rtd_convert_temperature ( rtd_t *ctx, uint16_t input_data, uint16_t referent_resistance )
{
    float float_value;
    float coefficient;
    coefficient = ( referent_resistance / 400.0 );

    input_data >>= 1;
    float_value = (float)input_data * coefficient;
    float_value /= 32;
    float_value -= 256;

    return float_value;
}

// ------------------------------------------------------------------------- END

